(define (list-ref l n) (nth n l))

(define (get-total-length img pathnum pathnamelist)
  (let* ((i 0)
		 (point-num 0)
		 (path-num pathnum)
		 (path-l pathnamelist)
		 )
	(while (< i path-num)
		   (set! point-num (+ point-num
							  (list-ref (gimp-path-get-points img (list-ref path-l i)) 2)
							  )
				 )
		   (set! i (+ i 1))
		   )
	point-num
	)
  )

(define (get-po-array img list n)
  (list-ref (gimp-path-get-points img (list-ref list n)) 3)
  )

;; cons-array to allocate proper size vector
(define (get-merged-path-data img num plist)
  (let* ((i 0) ;; index in a path list
		 (j 0) ;; index of points in path
		 (k 0) ;; index in the merged path data
		 (path-num num)
		 (path-l plist)
		 (size (* 3 (get-total-length img path-num path-l))) ;; number of floats
		 (parray (get-po-array img path-l i))
		 (data (cons-array size 'double))
		 )
	(while (< i path-num)
		   (set! parray (get-po-array img path-l i))
		   (set! len (length parray))
		   (set! j 0)
		   (while (< j len)
				  (aset data k (aref parray j))
				  (set! k (+ k 1))
				  (set! j (+ j 1))
				  (aset data k (aref parray j))
				  (set! k (+ k 1))
				  (set! j (+ j 1))
				  (if (and (not (eq? i 0)) (< j 3)) ;; disconnected component
					  (aset data k 3.0)
					  (aset data k (aref parray j))
					  )
				  (set! k (+ k 1))
				  (set! j (+ j 1))
				  )
		   (set! i (+ i 1))
		   )
	data ;; this returns the merged path data.
	)
  )

(define (get-locked-pathname-list img)
  (let* ((i 0)
		 (plist (cadr (gimp-path-list img)))
		 (size (length plist))
		 (return ())
		 )
	(while (< i size)
		   (if (eq? 1 (car (gimp-path-get-locked img (nth i plist))))
			   (set! return (cons (nth i plist) return)))
		   (set! i (+ i 1)))
	return))


(define (script-fu-merge-locked-paths img drw)
  (if (not (eq? (cadr (gimp-path-list img)) ()))
	  (let* ((pathname-list (cadr (gimp-path-list img)))
			 (pathnum 0)
			 (i 0)
			 )
		(set! pathname-list (get-locked-pathname-list img))
		(set! pathnum (length pathname-list))
		(gimp-path-set-points
		 img
		 (string-append (car pathname-list) " merged")
		 1
		 (get-total-length img pathnum pathname-list)
		 (get-merged-path-data img pathnum pathname-list)
		 )
		)
	  )
  )

(script-fu-register
 "script-fu-merge-locked-paths"
 "<Image>/Path ops./Merge locked paths."
 "Merge locked paths.
If you dont know what you are doing, Do not use this with open curves.
You are warned.
Use toggle button to merge all paths. it merges locked paths by default."
 "FUJITA Yuji (yuji@bijutsubu.com)"
 "FUJITA Yuji"
 "2000/06/10"
 "*"
 SF-IMAGE "Image to draw" 0
 SF-DRAWABLE "Drawable to edit" 0
)
