;; first appeared on 4/1, 2000.
;; 4/10, 2000. added error handling in case there is no paths.
;; 2000/06/10  changed to rotate all locked paths.

(define (get-locked-pathname-list img)
  (let* ((i 0)
		 (plist (cadr (gimp-path-list img)))
		 (size (length plist))
		 (return ())
		 )
	(while (< i size)
		   (if (eq? 1 (car (gimp-path-get-locked img (nth i plist))))
			   (set! return (cons (nth i plist) return)))
		   (set! i (+ i 1)))
	return))

(define (list-ref l n) (nth n l))
(define (dist x1 x2 y1 y2)
  (sqrt (+
		 (* (- x1 x2) (- x1 x2))
		 (* (- y1 y2) (- y1 y2))
		 )
		)
  )

;;  r for radian
(define (rotate-path-data path x y r)
  (let* ((i 0)
		 (size (length path))
		 (rpath path)
		 (ox 0)
		 (nx 0)
		 (oy 0)
		 (ny 0)
		 )
	(while (< i size)
		   (set! ox (aref path i))
		   (set! oy (aref path (+ i 1)))
		   (set! nx (+ x
					   (-
						(* (cos r) (- ox x))
						(* (sin r) (- oy y))
						)
					   )
				 )
		   (set! ny (+ y
					   (+
						(* (sin r) (- ox x))
						(* (cos r) (- oy y))
						)
					   )
				 )
		   (aset rpath i nx)
		   (set! i (+ i 1))
		   (aset rpath i ny)
		   (set! i (+ i 1))
		   (aset rpath i (aref path i))
		   (set! i (+ i 1))
		   )
  	rpath))
	

(define (path-rotate img pathn angle)
  (let* ((current-pathname pathn)
		 (path-data (gimp-path-get-points img current-pathname))
		 (path (list-ref path-data 3))
		 (rad (/ (* angle 3.14159265358979) 180))
		 (x (/ (car (gimp-image-width img)) 2))
		 (y (/ (car (gimp-image-height img)) 2))
		 (ang "hoge")
		 )
	(print-to-string angle ang)
    (gimp-path-set-points img
						  (string-append current-pathname " rotated" ang)
						  (list-ref path-data 0)
						  (list-ref path-data 2)
						  (rotate-path-data path x y rad)
						  )))

(define (script-fu-path-rotate img drw all? angle)
  (if (not (eq? (cadr (gimp-path-list img)) ))
	  (let* ((i 0)
			 (size 0)
			 (plist ()))
		(if (eq? all? TRUE)
			(set! plist (cadr (gimp-path-list img)))
			(if (eq? (get-locked-pathname-list img) ())
				(set! plist (cons (car (gimp-path-get-current img)) ()))
				(set! plist (get-locked-pathname-list img))))
		(set! size (length plist))
		(while (< i size)
			   (path-rotate img (nth i plist) angle)
			   (set! i (+ i 1))))))


(script-fu-register
 "script-fu-path-rotate"
 "<Image>/Path ops./Rotate locked paths."
 "Rotate locked paths. Pivot point is located to the center of the Image.
Positive value yields clockwise rotation.
If there is no locked paths, current path is operated."
 "FUJITA Yuji (yuji@wl.me.titech.ac.jp)"
 "FUJITA Yuji"
 "2000/06/10"
 "*"
 SF-IMAGE "Image to draw" 0
 SF-DRAWABLE "Drawable to edit" 0
 SF-TOGGLE "all?" FALSE
 SF-ADJUSTMENT "Angle" '(0.0 -180.0 180.0 1 1 1 1)
 )
